################################################################################
# DNA methylation analysis with RnBeads
# Epigenomics 2016 Workshop
# ------------------------------------------------------------------------------
#  RnBeads functionality beyond DNA methylation analysis
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# created: 2016-01-15
# author:  Fabian Mueller <rnbeads@mpi-inf.mpg.de>
# http://rnbeads.mpi-inf.mpg.de/
################################################################################

################################################################################
# (0) Preliminaries
################################################################################
# load the package
library(RnBeads)

# define the directory structure
# setwd(".")
resultDir <- file.path(getwd(), "results")

################################################################################
# (1) Logging
################################################################################
# initialize logger
logger.start("Logging tutorial", fname=c(NA, file.path(resultDir, "logfile.log")))
logger.isinitialized()

logger.start("Some tedious task")
logger.completed()

f <- function(){
    logger.start("Another tedious task")
    Sys.sleep(2)
        logger.start("Some tedious subtask")
        Sys.sleep(2)
        logger.completed()
        logger.start("Another, more tedious subtask")
        Sys.sleep(2)
            logger.start("Some tedious subsubtask")
            Sys.sleep(2)
            logger.completed()
            logger.start("Some even more tedious subsubtask")
            Sys.sleep(3)
            logger.completed()
        logger.completed()
    logger.completed()
}
f()

# Message types:
# STATUS : where in the execution am I? Typically static across program executions
# INFO   : program settings, etc. Typically setting-dependant
# WARNING: something did not go as expected.
# ERROR  : something went terribly wrong. Causes an error to be thrown

draw.lotto <- function(count=6) {
	urn <- 1:49
	primes <- c(2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47)
	logger.start(c("Drawing", count, "numbers from", length(urn)))
	for (i in 1:count) {
		if (i > 9) {
			msg <- "Too many numbers drawn"
			logger.error(msg, terminate=FALSE)
			stop(msg)
		}
		drawn <- sample(urn, 1)
		urn <- setdiff(urn, drawn)
		logger.info(c("The next number is", drawn))
		if (drawn %in% primes) {
			logger.warning("A prime number was drawn")
		}
	}
	logger.completed()
}
draw.lotto()
draw.lotto(15)

################################################################################
# (2) HTML report generation
################################################################################
# create and initialize the report
reportDir <- file.path(resultDir, "RnBeads_report")
rnb.initialize.reports(reportDir)
report <- createReport(file.path(reportDir, "myreport.html"), "An Eye Opener",
    page.title="Fascinating Report", authors=c("Me", "You"))
str(report)

# adding sections and subsections
stext <- "Here is some text for our awesome report"
report <- rnb.add.section(report, "Adding a text section", stext)
lorem1 <- c("Lorem ipsum dolor sit amet, consectetur adipiscing elit.
            Maecenas vestibulum placerat lobortis. Ut viverra fringilla
            urna at rutrum. In hac habitasse platea dictumst. Vestibulum
            adipiscing rutrum libero et interdum. Etiam sed odio ac nibh
            ullamcorper congue. Proin ac ipsum elit. Ut porta lorem sed
            lorem molestie pharetra.",
            "Vestibulum ante ipsum primis in faucibus orci luctus et
            ultrices posuere cubilia Curae; Cras ac augue eu turpis
            dignissim aliquam. Vivamus at arcu ligula, vel scelerisque nisi.
            Vivamus ac lorem libero, quis venenatis metus. Fusce et lectus
            at lectus vestibulum faucibus ac id sem.")
report <- rnb.add.section(report, "A subsection", lorem1, level=2)
lorem2 <- "Nunc congue molestie fringilla. Aliquam erat volutpat.
           Integer consequat turpis nec dolor pulvinar et vulputate magna
           adipiscing. Curabitur purus dolor, porttitor vel dapibus quis,
           eleifend at lacus. Cras at mauris est, quis aliquam libero.
           Nulla facilisi. Nam facilisis placerat aliquam. Morbi in odio
           non ligula mollis rhoncus et et erat. Maecenas ut dui nisl.
           Mauris consequat cursus augue quis euismod."

# adding paragraphs
rnb.add.paragraph(report, lorem2)
rnb.add.paragraph(report, "TODO: Add content here", paragraph.class="task")
rnb.add.paragraph(report, "To be or not to be, that is the question",
    paragraph.class="note")

report <- rnb.add.section(report, "Lists and Tables", "")
# lists
ll <- lapply(LETTERS[1:10], function(x) { paste(rep(x, 3), collapse="") })
ll
rnb.add.list(report, ll, type="o")
# tables
tt <- matrix(sapply(LETTERS[1:24],
    function(x) { paste(rep(x, 3), collapse="") }), ncol=4)
colnames(tt) <- paste("Col", 1:4)
rownames(tt) <- paste("Row", 1:6)
tt
rnb.add.table(report, tt, row.names=TRUE, first.col.header=TRUE,
    tcaption="A table")

# HTML can be embedded
stext <- c('<p>Some German umlauts:
                <ul>
                    <li>&Auml;</li>
                    <li>&Ouml;</li>
                    <li>&Uuml;</li>
                </ul>
            </p>',
            '<p> A link: <a href="http://rnbeads.mpi-inf.mpg.de/">
            RnBeads website</a></p>')
report <- rnb.add.section(report, "HTML code", stext)

# plots and figures
report <- rnb.add.section(report, "Plots", "")
report.plot <- createReportPlot("hist_normal", report, create.pdf=TRUE,
    high.png=200)
hist(rnorm(1000), breaks=50, col="blue")
off(report.plot)
desc <- 'A histogramm of samples drawn from the normal distribution
         Click <a href="myreport_pdfs/hist_normal.pdf">
         here</a> for the pdf version.'
report <- rnb.add.figure(report, desc, report.plot)

# plots with different parameter settings
plotSine <- function(a, b, fname) {
    report.plot <- createReportPlot(fname, report, create.pdf=FALSE,
        high.png=200)
    curve(sin(a*x)+b, -2*pi, 2*pi, ylim=c(-2, 2), col="blue", lwd=2)
    return(off(report.plot))
}

period.params <- c(a05=0.5, a1=1, a2=2)
intercept.params <- c(im1=-1, i0=0, i1=1)
plot.list <- list()
for (aa in names(period.params)){
    for (bb in names(intercept.params)){
        fname <- paste("sinePlot", aa, bb, sep="_")
        current.plot <- plotSine(period.params[aa], intercept.params[bb],
            fname)
        plot.list <- c(plot.list, current.plot)
    }
}
setting.names <- list('period parameter'=period.params,
    'intercept parameter'=intercept.params)
description <- 'Sine for various parameters'
report <- rnb.add.figure(report, description, plot.list, setting.names,
     selected.image=5)

# finishing and closing the report
off(report)
